@@ -77,25 +77,39 @@ class LinguisticVariable(object):
 		return min(mins), max(maxs)
 
 
-	def draw(self, ax, TGT=None):
+	def draw(self, ax, TGT=None, highlight=None):
 		"""
 		This method returns a matplotlib ax, representing all fuzzy sets contained in the liguistic variable.
 
 		Args:
 			ax: the axis to plot to.
-			TGT: show the memberships of a specific element of discourse TGT in the figure. 
+			TGT: show the memberships of a specific element of discourse TGT in the figure.
+			highlight: string, indicating the linguistic term/fuzzy set to highlight in the plot.
 		Returns:
 			A matplotlib axis, representing all fuzzy sets contained in the liguistic variable.
 		"""
 		mi, ma = self.get_universe_of_discourse()
 		x = linspace(mi, ma, 10000)
 
-		linestyles= ["-", "--", ":", "-."]
+		
+		if highlight is None:
+			linestyles= ["-", "--", ":", "-."]
+		else:
+			linestyles= ["-"]*4
+
 
 		for nn, fs in enumerate(self._FSlist):
 			if fs._type == "function":
 				y = [fs.get_value(xx) for xx in x]
-				ax.plot(x,y, linestyles[nn%4], label=fs._term, )
+				color = None
+				lw = 1
+
+				if highlight==fs._term: 
+					color="red"
+					lw =5 
+				elif highlight is not None:
+					color="lightgray"
+				ax.plot(x,y, linestyles[nn%4], lw=lw, label=fs._term, color=color)
 			else:
 				sns.regplot(fs._points.T[0], fs._points.T[1], marker="d", color="red", fit_reg=False, ax=ax)
 				f = interp1d(fs._points.T[0], fs._points.T[1], bounds_error=False, fill_value=(fs.boundary_values[0], fs.boundary_values[1]))
@@ -104,7 +118,7 @@ class LinguisticVariable(object):
 					ax.plot(TGT, f(TGT), "*", ms=10, label="x")
 		ax.set_xlabel(self._concept)
 		ax.set_ylabel("Membership degree")
-		ax.legend(loc="best")
+		if highlight is None: ax.legend(loc="best")
 		return ax
 
 
@@ -188,9 +202,9 @@ class FuzzySystem(object):
 		Creates a new fuzzy system.
 
 		Args:
-			operators: a list of strings, specifying fuzzy operators to be used instead of defaults.
-			Currently supported operators: 'AND_PRODUCT'.
+			operators: a list of strings, specifying fuzzy operators to be used instead of defaults. Currently supported operators: 'AND_PRODUCT'.
 			show_banner: True/False, toggles display of banner.
+			sanitize_input: sanitize variables' names to eliminate non-accepted characters (under development).
 			verbose: True/False, toggles verbose mode.
 	"""
 
@@ -202,7 +216,9 @@ class FuzzySystem(object):
 		self._crispvalues = {}
 		self._outputfunctions = {}
 		self._outputfuzzysets = {}
-		if show_banner: self._banner()
+
+		self._constants = []
+		
 		self._operators = operators
 
 		self._detected_type = None
@@ -211,6 +227,8 @@ class FuzzySystem(object):
 		if sanitize_input and verbose:
 			print (" * Warning: Simpful rules sanitization is enabled, please pay attention to possible collisions of symbols.")
 
+		if show_banner: self._banner()
+
 	def _banner(self):
 		import pkg_resources
 		vrs = pkg_resources.get_distribution('simpful').version 
@@ -220,7 +238,7 @@ class FuzzySystem(object):
 		print(" (____/(__)\\_)(_/(__)  (__)  \\____/\\____/")
 		print()
 		print(" Created by Marco S. Nobile (m.s.nobile@tue.nl)")
-		print(" and Simone Spolaor (simone.spolaor@disco.unimib.it)")
+		print(" and Simone Spolaor (simone.spolaor@unimib.it)")
 		print()
 
 
@@ -241,6 +259,23 @@ class FuzzySystem(object):
 		except ValueError:
 			raise Exception("ERROR: specified value for "+name+" is not an integer or float: "+value)
 
+	def set_constant(self, name, value, verbose=False):
+		"""
+		Sets the numerical value of a linguistic variable to a constant value (i.e. ignore fuzzy inference).
+
+		Args:
+			name: name of the linguistic variables to be set to a constant value.
+			value: numerical value to be set.
+			verbose: True/False, toggles verbose mode.
+		"""
+		if self._sanitize_input: name = self._sanitize(name)
+		try: 
+			value = float(value)
+			self._variables[name] = value
+			self._constants.append(name)
+			if verbose: print(" * Variable %s set to a constant value %f" % (name, value))
+		except ValueError:
+			raise Exception("ERROR: specified value for "+name+" is not an integer or float: "+value)
 
 	def add_rules_from_file(self, path, verbose=False):
 		"""
@@ -320,7 +355,7 @@ class FuzzySystem(object):
 		Args:
 			name: string containing the identifying name of the output function.
 			function: string containing the output function to be added to the fuzzy system.
-			The function specified in the string must use the names of linguistic variables contained in the fuzzy system object.
+				The function specified in the string must use the names of linguistic variables contained in the fuzzy system object.
 			verbose: True/False, toggles verbose mode.
 		"""
 		if self._sanitize_input: name = self._sanitize(name)
@@ -381,7 +416,12 @@ class FuzzySystem(object):
 					else:
 						string_to_evaluate = self._outputfunctions[outterm]
 						for k,v in self._variables.items():
-							string_to_evaluate = string_to_evaluate.replace(k,str(v))
+							# old version
+							# string_to_evaluate = string_to_evaluate.replace(k,str(v))
+
+							# match a variable name preceeded or followed by non-alphanumeric and _ characters
+							# substitute it with its numerical value
+							string_to_evaluate = re.sub(r"(?P<front>\W|^)"+k+r"(?P<end>\W|$)", r"\g<front>"+str(v)+r"\g<end>", string_to_evaluate)
 						crispvalue = eval(string_to_evaluate)						
 
 					try:
@@ -487,15 +527,32 @@ class FuzzySystem(object):
 		Returns:
 			a dictionary, containing as keys the variables' names and as values their numerical inferred values.
 		"""
-		# default: inference on ALL rules/terms
 		if self._sanitize and terms is not None: 
 			terms = [self._sanitize(term) for term in terms]
+		
+		# default: inference on ALL rules/terms
 		if terms == None:
 			temp = [rule[1][0] for rule in self._rules] 
-			terms= list(set(temp))
+			terms = list(set(temp))
+		else:
+			# get rid of duplicates in terms to infer
+			terms = list(set(terms))
+			for t in terms:
+				if t not in set([rule[1][0] for rule in self._rules]):
+					raise Exception("ERROR: Variable "+t+" does not appear in any consequent.")
 
 		array_rules = array(self._rules, dtype='object')
-		result = self.mediate(terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors)
+		if len(self._constants)==0:
+			result = self.mediate(terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors)
+		else:
+			#remove constant variables from list of variables to infer
+			ncost_terms = [t for t in terms if t not in self._constants]
+			result = self.mediate(ncost_terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors)
+			#add values of constant variables
+			cost_terms = [t for t in terms if t in self._constants]
+			for name in cost_terms:
+				result[name] = self._variables[name]
+		
 		return result
 
 
@@ -512,22 +569,39 @@ class FuzzySystem(object):
 		Returns:
 			a dictionary, containing as keys the variables' names and as values their numerical inferred values.
 		"""
-
-		# default: inference on ALL rules/terms
 		if self._sanitize and terms is not None: 
 			terms = [self._sanitize(term) for term in terms]
+		
+		# default: inference on ALL rules/terms
 		if terms == None:
 			temp = [rule[1][0] for rule in self._rules] 
 			terms= list(set(temp))
+		else:
+			# get rid of duplicates in terms to infer
+			terms = list(set(terms))
+			for t in terms:
+				if t not in set([rule[1][0] for rule in self._rules]):
+					raise Exception("ERROR: Variable "+t+" does not appear in any consequent.")
 
 		array_rules = array(self._rules, dtype=object)
-		result = self.mediate_Mamdani( terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors, verbose=verbose , subdivisions=subdivisions)
+		if len(self._constants)==0:
+			result = self.mediate_Mamdani(terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors, verbose=verbose , subdivisions=subdivisions)
+		else:
+			#remove constant variables from list of variables to infer
+			ncost_terms = [t for t in terms if t not in self._constants]
+			result = self.mediate_Mamdani(ncost_terms, array_rules.T[0], array_rules.T[1], ignore_errors=ignore_errors, verbose=verbose , subdivisions=subdivisions)
+			#add values of constant variables
+			cost_terms = [t for t in terms if t in self._constants]
+			for name in cost_terms:
+				result[name] = self._variables[name]
+
 		return result
 
 
 	def probabilistic_inference(self, terms=None, ignore_errors=False, verbose=False):
 		raise NotImplementedError()
 
+
 	def inference(self, terms=None, ignore_errors=False, verbose=False, subdivisions=1000):
 		"""
 		Performs the fuzzy inference, trying to automatically choose the correct inference engine.
@@ -551,8 +625,7 @@ class FuzzySystem(object):
 			raise Exception("ERROR: simpful could not detect the model type, please use either Sugeno_inference() or Mamdani_inference() methods.")
 			
 
-
-	def produce_figure(self, outputfile='output.pdf'):
+	def produce_figure(self, outputfile='output.pdf', max_figures_per_row=4):
 		"""
 		Plots the membership functions of each linguistic variable contained in the fuzzy system.
 
@@ -564,9 +637,12 @@ class FuzzySystem(object):
 
 		num_ling_variables = len(self._lvs)
 		#print(" * Detected %d linguistic variables" % num_ling_variables)
-		columns = min(num_ling_variables, 4)
-		if num_ling_variables>4:
-			rows = num_ling_variables//4 + 1
+		columns = min(num_ling_variables, max_figures_per_row)
+		if num_ling_variables>max_figures_per_row:
+			if num_ling_variables%max_figures_per_row==0:
+				rows = num_ling_variables//max_figures_per_row 
+			else:
+				rows = num_ling_variables//max_figures_per_row + 1
 		else:
 			rows = 1
 
@@ -577,15 +653,15 @@ class FuzzySystem(object):
 
 		n = 0
 		for k, v in self._lvs.items():
-			r = n%4
-			c = n//4
+			r = n%max_figures_per_row
+			c = n//max_figures_per_row
 			v.draw(ax[c][r])
 			ax[c][r].set_ylim(0,1)
 			n+=1
 
 		for m in range(n, columns*rows):
-			r = m%4
-			c = m//4
+			r = m%max_figures_per_row
+			c = m//max_figures_per_row
 			ax[c][r].axis('off')
 
 		fig.tight_layout()
@@ -593,7 +669,16 @@ class FuzzySystem(object):
 
 
 	def aggregate(self, list_variables, function):
-		# TEST
+		"""
+		Performs a fuzzy aggregation of linguistic variables contained in a FuzzySystem object.
+
+		Args:
+			list_variables: list of linguistic variables names in the FuzzySystem object to aggregate.
+			function: pointer to an aggregation function. The function must accept as an argument a list of membership values.
+
+		Returns:
+			the aggregated membership values.
+		""" 
 		memberships = []
 		for variable, fuzzyset in list_variables.items():
 			value = self._variables[variable]
